
import numpy as np
import json
from sensiml.dclproj import  DataSegment, DataSegments

def get_info_json() -> str:
    return json.dumps(get_info())

def get_info() -> dict:
    return {
        "name": "Symmetric Moving Average",
        "type": "Transform",
        "subtype": "Filter",
        "description": "Performs a symmetric moving average filter on the input signal with a filter length of filter order 2*l+1",
        "input_contract": [
            {
                "name": "columns",
                "display_name":"Input Columns",
                "type": "list",
                "num_columns": 1
            },
            {
                "name": "filter_order",
                "display_name":"Filter Order",
                "type": "int",
                "default": 10,
            },
            {
                "name": "normalize",
                "display_name":"Normalize",
                "type": "bool",
                "default": False,
            },
        ],
        "output_contract": [{}],
    }
    
def validate_params(input_contract, params: dict) -> bool:
    
    params = json.loads(params)
    
    for ic_param in input_contract:
        if ic_param["name"] not in params:
            raise Exception(f"param {ic_param['name']} is required")

    return params


def to_data_studio_dataseries(name, data, time=None):
    
    if time is None:
        time = list(range(len(data)))
    
    if len(time) != len(data):
        raise Exception(f"Time and Data are not the same length {len(time)} {len(data)}")
        

    return {name: (time, data.tolist())}


def convert_to_datasegments(data: dict, dtype=np.float32) -> DataSegments:

    
    converted_data = {}
    for k, v in dict(data).items():
        if not k:
            continue
        converted_data[k] = {"X":[float(item) for item in v[0]],
                             "Y":[float(item) for item in v[1]]}
               
    return converted_data


def transform_data(data_segments, params):

    input_column = params['columns'][0]
    
    data = np.array(data_segments[input_column]['Y'])
    
    if params['normalize']:
        data = normalize(data)
        
    return symmetric_ma_filter(data, params['filter_order'])
   

def transform(data, params):    
    
    data_segments = convert_to_datasegments(data)
    
    params = validate_params(get_info()['input_contract'], params)
    
    result =  transform_data(data_segments, params)
    
    converted_result = to_data_studio_dataseries(params['columns'][0], result)
    
    return converted_result
    

def symmetric_ma_filter(data, filter_order=3):
    """
    Performs a symmetric moving average filter on the input column,
    creates a new column with the filtered data.

    Args:
        data: DataFrame containing the time series data.
        filter order: the number of samples to average to the left and right.

    Returns:
        input data after having been passed through symmetric moving average filter
    """
    
    ma_filtered = np.zeros(len(data))

    for i in range(len(data)):
        if i < filter_order:
            ma_filtered[i] = data[i : i + filter_order + 1].mean()
        elif i > (len(data) - filter_order):
            ma_filtered[i] = data[i - filter_order :].mean()
        else:
            ma_filtered[i] = data[i - filter_order : i + filter_order + 1].mean()

    return np.round(ma_filtered)



def normalize(data):
    """
    Normalizes the sensor data

    Args:
        data: DataFrame containing the time series data.
        filter order: the number of samples to average to the left and right.

    Returns:
        input data after having been passed through symmetric moving average filter
    """
    
    return (data - data.mean())/np.std(data)